# Author: Mathieu Blondel
# License: BSD

import numpy as np

from sklearn.utils import check_random_state
from sklearn.preprocessing import LabelBinarizer
from six.moves import xrange

from .base import BaseClassifier, BaseRegressor
from .dataset_fast import get_dataset
from .sdca_fast import _prox_sdca_fit


class _BaseSDCA(object):

    def _get_alpha2_lasso(self, y, alpha1):
        if self.loss == "squared":
            y_bar = 0.5 * np.mean(y ** 2)

        elif self.loss == "absolute":
            y_bar = np.mean(np.abs(y))

        elif self.loss in ("hinge", "squared_hinge"):
            y_bar = 1.0

        elif self.loss == "smooth_hinge":
            if self.gamma < 1:
                y_bar = 1 - 0.5 * self.gamma
            else:
                y_bar = 0.5 / self.gamma

        else:
            raise ValueError("Unknown loss.")

        return  self.tol * (alpha1 / y_bar) ** 2

    def _fit(self, X, Y):
        n_samples, n_features = X.shape
        n_vectors = Y.shape[1]

        ds = get_dataset(X, order="c")
        self.coef_ = np.zeros((n_vectors, n_features), dtype=np.float64)
        self.dual_coef_ = np.zeros((n_vectors, n_samples), dtype=np.float64)

        alpha1 = self.l1_ratio * self.alpha
        alpha2 = (1 - self.l1_ratio) * self.alpha

        if self.loss == "squared_hinge":
            # For consistency with the rest of lightning.
            alpha1 *= 0.5
            alpha2 *= 0.5

        tol = self.tol
        n_calls = n_samples if self.n_calls is None else self.n_calls
        rng = check_random_state(self.random_state)
        loss = self._get_loss()

        for i in xrange(n_vectors):
            y = Y[:, i]

            if self.l1_ratio == 1.0:
                # Prox-SDCA needs a strongly convex regularizer so adds some
                # L2 penalty (see paper).
                alpha2 = self._get_alpha2_lasso(y, alpha1)
                tol = self.tol * 0.5

            _prox_sdca_fit(self, ds, y, self.coef_[i], self.dual_coef_[i],
                           alpha1, alpha2, loss, self.gamma, self.max_iter,
                           tol, self.callback, n_calls, self.verbose, rng)

        return self


class SDCAClassifier(BaseClassifier, _BaseSDCA):
    """
    Estimator for learning linear classifiers by (proximal) SDCA.

    Solves the following objective:

        minimize_w  1 / n_samples * \sum_i loss(w^T x_i, y_i)
                    + alpha * l1_ratio * ||w||_1
                    + alpha * (1 - l1_ratio) * 0.5 * ||w||^2_2

    Parameters
    ----------
    loss: string, {'squared', 'absolute', 'hinge', 'smooth_hinge', 'squared_hinge'}
         Loss function to use in the model.
    alpha: float
         Amount of regularization (see model formulation above).
    l1_ratio: float
         Ratio between the L1 and L2 regularization (see model formulation above).
    gamma : float
        gamma parameter in the "smooth_hinge" loss (not used for other
        loss functions)
    tol : float
        stopping criterion tolerance.
    max_iter : int
        maximum number of outer iterations (also known as epochs).
    verbose : int
        verbosity level. Set positive to print progress information.
    callback : callable or None
        if given, callback(self) will be called on each outer iteration
        (epoch).
    random_state: int or RandomState
        Pseudo-random number generator state used for random sampling.
    """

    def __init__(self, alpha=1.0, l1_ratio=0, loss="hinge", gamma=1.0,
                 max_iter=100, tol=1e-3, callback=None, n_calls=None, verbose=0,
                 random_state=None):
        self.alpha = alpha
        self.l1_ratio = l1_ratio
        self.loss = loss
        self.gamma = gamma
        self.max_iter = max_iter
        self.tol = tol
        self.callback = callback
        self.n_calls = n_calls
        self.verbose = verbose
        self.random_state = random_state

    def _get_loss(self):
        losses = {
            "squared": 0,
            "absolute": 1,
            "hinge": 2,
            "smooth_hinge": 3,
            "squared_hinge": 4,
        }
        return losses[self.loss]

    def fit(self, X, y):
        self._set_label_transformers(y)
        Y = np.asfortranarray(self.label_binarizer_.transform(y),
                              dtype=np.float64)
        return self._fit(X, Y)


class SDCARegressor(BaseRegressor, _BaseSDCA):
    """
    Estimator for learning linear regressors by (proximal) SDCA.

    Solves the following objective:

        minimize_w  1 / n_samples * \sum_i loss(w^T x_i, y_i)
                    + alpha * l1_ratio * ||w||_1
                    + alpha * (1 - l1_ratio) * 0.5 * ||w||^2_2
    Parameters
    ----------
    loss: string, {'squared', 'absolute'}
         Loss function to use in the model.
    alpha: float
         Amount of regularization (see model formulation above).
    l1_ratio: float
         Ratio between the L1 and L2 regularization (see model formulation above).
    tol : float
        stopping criterion tolerance.
    max_iter : int
        maximum number of outer iterations (also known as epochs).
    verbose : int
        verbosity level. Set positive to print progress information.
    callback : callable or None
        if given, callback(self) will be called on each outer iteration
        (epoch).
    random_state: int or RandomState
        Pseudo-random number generator state used for random sampling.
    """

    def __init__(self, alpha=1.0, l1_ratio=0, loss="squared",
                 max_iter=100, tol=1e-3, callback=None, n_calls=None, verbose=0,
                 random_state=None):
        self.alpha = alpha
        self.l1_ratio = l1_ratio
        self.loss = loss
        self.gamma = 1.0
        self.max_iter = max_iter
        self.tol = tol
        self.callback = callback
        self.n_calls = n_calls
        self.verbose = verbose
        self.random_state = random_state

    def _get_loss(self):
        losses = {
            "squared": 0,
            "absolute": 1,
        }
        return losses[self.loss]

    def fit(self, X, y):
        self.outputs_2d_ = len(y.shape) > 1
        Y = y.reshape(-1, 1) if not self.outputs_2d_ else y
        Y = Y.astype(np.float64)
        return self._fit(X, Y)



from .impl.datasets.loaders import get_data_home
from .impl.datasets.loaders import load_news20
from .impl.datasets.loaders import load_usps
from .impl.datasets.loaders import load_mnist
from .impl.datasets.loaders import get_loader
from .impl.datasets.loaders import load_dataset
from .impl.datasets.utils import make_ovo
from .impl.datasets.samples_generator import make_nn_regression
from .impl.datasets.samples_generator import make_classification



# Author: Mathieu Blondel
# License: BSD

cdef class LossFunction:

    cpdef double loss(self, double p, double y)
    cpdef double get_update(self, double p, double y)


cdef class ModifiedHuber(LossFunction):

    cpdef double loss(self, double p, double y)

    cpdef double get_update(self, double p, double y)


cdef class Hinge(LossFunction):

    cdef double threshold

    cpdef double loss(self, double p, double y)

    cpdef double get_update(self, double p, double y)


cdef class SmoothHinge(LossFunction):

    cdef double gamma

    cpdef double loss(self, double p, double y)

    cpdef double get_update(self, double p, double y)


cdef class SquaredHinge(LossFunction):

    cdef double threshold

    cpdef double loss(self, double p, double y)

    cpdef double get_update(self, double p, double y)


cdef class Log(LossFunction):

    cpdef double loss(self, double p, double y)

    cpdef double get_update(self, double p, double y)


cdef class SquaredLoss(LossFunction):

    cpdef double loss(self, double p, double y)

    cpdef double get_update(self, double p, double y)


cdef class Huber(LossFunction):

    cdef double c

    cpdef double loss(self, double p, double y)

    cpdef double get_update(self, double p, double y)


cdef class EpsilonInsensitive(LossFunction):

    cdef double epsilon

    cpdef double loss(self, double p, double y)

    cpdef double get_update(self, double p, double y)




import numpy as np
import scipy.sparse as sp

from sklearn.metrics.pairwise import linear_kernel
from sklearn.datasets.samples_generator import make_regression
from six.moves import xrange

from sklearn.utils.testing import assert_equal
from sklearn.utils.testing import assert_greater
from sklearn.utils.testing import assert_array_almost_equal

from lightning.impl.datasets.samples_generator import make_classification
from lightning.impl.dual_cd import LinearSVC
from lightning.impl.dual_cd import LinearSVR
from lightning.impl.dual_cd_fast import sparse_dot
from lightning.impl.dataset_fast import get_dataset

bin_dense, bin_target = make_classification(n_samples=200, n_features=100,
                                            n_informative=5,
                                            n_classes=2, random_state=0)
bin_csr = sp.csr_matrix(bin_dense)

mult_dense, mult_target = make_classification(n_samples=300, n_features=100,
                                              n_informative=5,
                                              n_classes=3, random_state=0)
mult_sparse = sp.csr_matrix(mult_dense)

reg_dense, reg_target = make_regression(n_samples=200, n_features=100,
                                        n_informative=5, random_state=0)


def test_sparse_dot():
    for data in (bin_dense, bin_csr):
        K = linear_kernel(data)
        K2 = np.zeros_like(K)
        ds = get_dataset(data)

        for i in xrange(data.shape[0]):
            for j in xrange(i, data.shape[0]):
                K2[i, j] = sparse_dot(ds, i, j)
                K2[j, i] = K[i, j]

    assert_array_almost_equal(K, K2)


def test_fit_linear_binary():
    for data in (bin_dense, bin_csr):
        for loss in ("l1", "l2"):
            clf = LinearSVC(loss=loss, random_state=0, max_iter=10)
            clf.fit(data, bin_target)
            assert_equal(list(clf.classes_), [0, 1])
            assert_equal(clf.score(data, bin_target), 1.0)
            y_pred = clf.decision_function(data).ravel()


def test_fit_linear_binary_auc():
    for data in (bin_dense, bin_csr):
        for loss in ("l1", "l2"):
            clf = LinearSVC(loss=loss, criterion="auc", random_state=0,
                            max_iter=25)
            clf.fit(data, bin_target)
            assert_equal(clf.score(data, bin_target), 1.0)


def test_fit_linear_multi():
    for data in (mult_dense, mult_sparse):
        clf = LinearSVC(random_state=0)
        clf.fit(data, mult_target)
        assert_equal(list(clf.classes_), [0, 1, 2])
        y_pred = clf.predict(data)
        acc = np.mean(y_pred == mult_target)
        assert_greater(acc, 0.85)


def test_warm_start():
    clf = LinearSVC(warm_start=True, loss="l1", random_state=0, max_iter=100)
    for C in (0.1, 0.2):
        clf.C = C

        clf.fit(bin_dense, bin_target)
        acc = clf.score(bin_dense, bin_target)
        assert_greater(acc, 0.99)


def test_linear_svr():
    reg = LinearSVR(random_state=0)
    reg.fit(reg_dense, reg_target)
    assert_greater(reg.score(reg_dense, reg_target), 0.99)


def test_linear_svr_fit_intercept():
    reg = LinearSVR(random_state=0, fit_intercept=True)
    reg.fit(reg_dense, reg_target)
    assert_greater(reg.score(reg_dense, reg_target), 0.99)


def test_linear_svr_l2():
    reg = LinearSVR(loss="l2", random_state=0)
    reg.fit(reg_dense, reg_target)
    assert_greater(reg.score(reg_dense, reg_target), 0.99)


def test_linear_svr_warm_start():
    reg = LinearSVR(C=1e-3, random_state=0, warm_start=True)
    reg.fit(reg_dense, reg_target)
    assert_greater(reg.score(reg_dense, reg_target), 0.96)

    reg.C = 1
    reg.fit(reg_dense, reg_target)
    assert_greater(reg.score(reg_dense, reg_target), 0.99)



import numpy as np

def make_ovo(X, y, class1, class2):
    classes = np.unique(y)

    if len(y) <= 2:
        return X, y

    c1 = classes[class1]
    c2 = classes[class2]
    cond = np.logical_or(y == classes[c1], y == classes[c2])
    y = y[cond]
    y[y == c1] = 0
    y[y == c2] = 1
    ind = np.arange(X.shape[0])

    return X[ind[cond]], y



import numpy as np
from nose.tools import assert_equal

from lightning.impl import prox_fast

def test_tv1_denoise():
    # test the prox of TV1D
    # since its not trivial to check the KKT conditions
    # we check that the proximal point algorithm converges
    # to a solution to the TV minimization
    n_iter = 100
    n_features = 100

    # repeat the test 10 times
    for nrun in range(10):
        x = np.random.randn(n_features)
        for _ in range(n_iter):
            prox_fast.prox_tv1d(x, 1.0)
        # check that the solution is flat
        np.testing.assert_allclose(x, x.mean() * np.ones(n_features))


def test_tv1d_dtype():
    # check that prox_tv1d preserve 32bit

    x = np.arange(5)
    for dtype in (np.float32, np.float64):
        y = x.astype(dtype, copy=True)
        prox_fast.prox_tv1d(y, 0.01)
        assert_equal(y.dtype, dtype)




from optparse import OptionParser
import pickle

from lightning.classification import CDClassifier
from lightning.classification import FistaClassifier
from lightning.classification import SGDClassifier


try:
    from svmlight_loader import load_svmlight_file
except ImportError:
    from sklearn.datasets import load_svmlight_file

parser = OptionParser("lightning_train [options] train_file model_file")

parser.add_option("", "--solver",
                  action="store", type="str", dest="solver", default="bcd-ls",
                  help="Solver to use (bcd-ls, bcd-cst, fista, sgd). "
                  "Default: bcd-ls.")

parser.add_option("", "--loss",
                  action="store", type="str", dest="loss",
                  default="squared_hinge",
                  help="Loss function to use (squared_hinge, log). "
                  "Default: squared_hinge.")

parser.add_option("", "--penalty",
                  action="store", type="str", dest="penalty",
                  default="l1/l2",
                  help="Penalty to use (l2, l1, l1/l2). Default: l1/l2.")

parser.add_option("", "--alpha",
                  action="store", type="float", dest="alpha",
                  default=1e-4,
                  help="Strength of the penalty. Default: 1e-4.")

parser.add_option("", "--ovr",
                  action="store_true", dest="ovr",
                  help="Whether to activate one-vs-rest instead of direct "
                  "multiclass loss.")

parser.add_option("", "--max-iter",
                  action="store", type="int", dest="max_iter", default=10,
                  help="Maximum number of outer iterations. Default: 10.")

parser.add_option("", "--tol",
                  action="store", type="float", dest="tol", default=0.05,
                  help="Tolerance of the stopping criterion. Default: 0.05.")

parser.add_option("", "--eta0",
                  action="store", type="float", dest="eta0", default=1e-3,
                  help="Step size (SGD only). Default: 1e-3.")

(opts, args) = parser.parse_args()

if len(args) != 2:
    parser.print_help()
    exit()

X, y = load_svmlight_file(args[0])

multiclass = not opts.ovr

if opts.solver.startswith("bcd"):
    if opts.solver == "bcd-cst":
        max_steps = 0
    else:
        max_steps = 30

    clf = CDClassifier(penalty=opts.penalty,
                       loss=opts.loss,
                       multiclass=multiclass,
                       max_iter=opts.max_iter,
                       max_steps=max_steps,
                       alpha=opts.alpha,
                       C=1.0 / X.shape[0],
                       tol=opts.tol)
elif opts.solver == "fista":
    clf = FistaClassifier(penalty=opts.penalty,
                          loss=opts.loss,
                          multiclass=multiclass,
                          max_iter=opts.max_iter,
                          alpha=opts.alpha,
                          C=1.0 / X.shape[0])
elif opts.solver == "sgd":
    clf = SGDClassifier(penalty=opts.penalty,
                        loss=opts.loss,
                        multiclass=multiclass,
                        max_iter=opts.max_iter,
                        alpha=opts.alpha,
                        learning_rate="constant",
                        eta0=opts.eta0)
else:
    print "Invalid solver"
    exit()

clf.fit(X, y)

print "Percentage of non-zero rows:"
print clf.n_nonzero(percentage=True)

pickle.dump(clf, file(args[1], "w"), pickle.HIGHEST_PROTOCOL)



"""
=============================
Kernel SVM by Newton's method
=============================

This module provides a kernel SVM solver using Newton's method.

"""
# Author: Mathieu Blondel
# License: BSD

import numpy as np

from scipy.sparse.linalg import cg
from scipy.linalg import solve

from sklearn.utils import safe_mask
from sklearn.preprocessing import LabelBinarizer
from sklearn.utils import check_random_state
from sklearn.metrics.pairwise import pairwise_kernels
from six.moves import xrange

from .base import BaseClassifier


class KernelSVC(BaseClassifier):
    """Estimator for learning kernel SVMs by Newton's method.

    Parameters
    ----------

    alpha : float
        Weight of the penalty term.

    solver : str, 'cg', 'dense'

    max_iter : int
        Maximum number of iterations to perform.

    tol : float
        Tolerance of the stopping criterion.

    kernel: "linear" | "poly" | "rbf" | "sigmoid" | "cosine" | "precomputed"
        Kernel to use.  Default: "linear"

    degree : int, default=3
        Degree for poly, rbf and sigmoid kernels. Ignored by other kernels.

    gamma : float, optional
        Kernel coefficient for rbf and poly kernels. Default: 1/n_features.
        Ignored by other kernels.

    coef0 : float, optional
        Independent term in poly and sigmoid kernels.
        Ignored by other kernels.

    random_state : RandomState or int
        The seed of the pseudo random number generator to use.

    verbose : int
        Verbosity level.

    n_jobs : int
        Number of jobs to use to compute the kernel matrix.

    Example
    -------

    >>> from sklearn.datasets import make_classification
    >>> from lightning.classification import KernelSVC
    >>> X, y = make_classification()
    >>> clf = KernelSVC().fit(X, y)
    >>> accuracy = clf.score(X, y)
    """

    def __init__(self, alpha=1.0, solver="cg",
                 max_iter=50, tol=1e-3,
                 kernel="linear", gamma=0.1, coef0=1, degree=4,
                 random_state=None, verbose=0, n_jobs=1):
        self.alpha = alpha
        self.solver = solver
        self.max_iter = max_iter
        self.tol = tol
        self.kernel = kernel
        self.gamma = gamma
        self.coef0 = coef0
        self.degree = degree
        self.random_state = random_state
        self.verbose = verbose
        self.n_jobs = n_jobs

    def _kernel_params(self):
        return {"gamma": self.gamma,
                "degree": self.degree,
                "coef0": self.coef0}

    def _solve(self, A, b):
        if self.solver == "cg":
            x, info = cg(A, b, tol=self.tol)
        elif self.solver == "dense":
            x = solve(A, b, sym_pos=True)
        return x

    def _fit_binary(self, K, y, rs):
        n_samples = K.shape[0]
        coef = np.zeros(n_samples)
        if n_samples < 1000:
            sv = np.ones(n_samples, dtype=bool)
        else:
            sv = np.zeros(n_samples, dtype=bool)
            sv[:1000] = True
            rs.shuffle(sv)

        for t in xrange(1, self.max_iter + 1):
            if self.verbose:
                print("Iteration", t, "#SV=", np.sum(sv))

            K_sv = K[sv][:, sv]
            I = np.diag(self.alpha * np.ones(K_sv.shape[0]))

            coef_sv = self._solve(K_sv + I, y[sv])

            coef *= 0
            coef[sv] = coef_sv
            pred = np.dot(K, coef)
            errors = 1 - y * pred
            last_sv = sv
            sv = errors > 0

            if np.array_equal(last_sv, sv):
                if self.verbose:
                    print("Converged at iteration", t)
                break

        return coef

    def _post_process(self, X):
        # We can't know the support vectors when using precomputed kernels.
        if self.kernel != "precomputed":
            sv = np.sum(self.coef_ != 0, axis=0, dtype=bool)
            if np.sum(sv) > 0:
                self.coef_ = np.ascontiguousarray(self.coef_[:, sv])
                mask = safe_mask(X, sv)
                self.support_vectors_ = np.ascontiguousarray(X[mask])
                self.support_indices_ = np.arange(X.shape[0],
                                                  dtype=np.int32)[sv]
                self.n_samples_ = X.shape[0]

            if self.verbose >= 1:
                print("Number of support vectors:", np.sum(sv))

    def fit(self, X, y):
        """Fit model according to X and y.

        Parameters
        ----------
        X : array-like, shape = [n_samples, n_features]
            Training vectors, where n_samples is the number of samples
            and n_features is the number of features.

        y : array-like, shape = [n_samples]
            Target values.

        Returns
        -------
        self : classifier
            Returns self.
        """
        n_samples, n_features = X.shape
        rs = check_random_state(self.random_state)

        self.label_binarizer_ = LabelBinarizer(neg_label=-1, pos_label=1)
        Y = self.label_binarizer_.fit_transform(y)
        self.classes_ = self.label_binarizer_.classes_.astype(np.int32)
        n_vectors = Y.shape[1]

        if self.verbose:
            print("Pre-computing kernel matrix...")

        K = pairwise_kernels(X, filter_params=True, n_jobs=self.n_jobs,
                             metric=self.kernel, **self._kernel_params())

        coef = [self._fit_binary(K, Y[:, i], rs) for i in xrange(n_vectors)]
        self.coef_ = np.array(coef)
        self.intercept_ = np.zeros(n_vectors, dtype=np.float64)

        self._post_process(X)

        return self

    def decision_function(self, X):
        """
        Return the decision function for test vectors X.

        Parameters
        ----------
        X : array-like, shape = [n_samples, n_features]

        Returns
        -------
        P : array, shape = [n_classes, n_samples]
            Decision function for X
        """
        K = pairwise_kernels(X, self.support_vectors_, filter_params=True,
                             n_jobs=self.n_jobs, metric=self.kernel,
                             **self._kernel_params())
        return np.dot(K, self.coef_.T)



# Author: Mathieu Blondel
# License: BSD
import os

import numpy as np
import scipy.sparse as sp

try:
    from svmlight_loader import load_svmlight_files
except ImportError:
    from sklearn.datasets import load_svmlight_files

from sklearn.datasets.base import get_data_home as _get_data_home
from sklearn.utils import check_random_state

try:
    from sklearn.model_selection import StratifiedShuffleSplit
except ImportError:
    from sklearn.cross_validation import StratifiedShuffleSplit


def get_data_home():
    return _get_data_home().replace("scikit_learn", "lightning")


def _load(train_file, test_file, name):
    if not os.path.exists(train_file) or \
       (test_file is not None and not os.path.exists(test_file)):
        raise IOError("Dataset missing! " +
                      "Run 'make download-%s' at the project root." % name)

    if test_file:
        return load_svmlight_files((train_file, test_file))
    else:
        X, y = load_svmlight_files((train_file,))
        return X, y, None, None


def _todense(data):
    X_train, y_train, X_test, y_test = data
    X_train = X_train.toarray()
    if X_test is not None:
        X_test = X_test.toarray()
    return X_train, y_train, X_test, y_test

# regression

def load_abalone():
    data_home = get_data_home()
    train_file = os.path.join(data_home, "abalone_scale")
    return _todense(_load(train_file, None, "abalone"))


def load_cadata():
    data_home = get_data_home()
    train_file = os.path.join(data_home, "cadata")
    return _todense(_load(train_file, None, "cadata"))


def load_cpusmall():
    data_home = get_data_home()
    train_file = os.path.join(data_home, "cpusmall_scale")
    return _todense(_load(train_file, None, "cpusmall"))


def load_space_ga():
    data_home = get_data_home()
    train_file = os.path.join(data_home, "space_ga_scale")
    return _todense(_load(train_file, None, "space_ga"))


def load_YearPredictionMSD():
    data_home = get_data_home()
    train_file = os.path.join(data_home, "YearPredictionMSD")
    test_file = os.path.join(data_home, "YearPredictionMSD.t")
    return _todense(_load(train_file, test_file, "YearPredictionMSD"))

# binary classification

def load_adult():
    data_home = get_data_home()
    train_file = os.path.join(data_home, "adult", "adult.trn")
    test_file = os.path.join(data_home, "adult", "adult.tst")
    return _todense(_load(train_file, test_file, "adult"))


def load_banana():
    data_home = get_data_home()
    train_file = os.path.join(data_home, "banana", "banana.all.txt")
    return _todense(_load(train_file, None, "banana"))


def load_covtype():
    data_home = get_data_home()
    train_file = os.path.join(data_home, "covtype.libsvm.binary.scale")
    return _todense(_load(train_file, None, "covtype"))


def load_covtype_subset():
    X_train, y_train, _, _ = load_covtype()
    cv = StratifiedShuffleSplit(y_train, train_size=100000, test_size=50000,
                                n_iterations=1, random_state=0)
    tr, te = iter(cv).next()
    return X_train[tr], y_train[tr], X_train[te], y_train[te]


def load_ijcnn():
    data_home = get_data_home()
    train_file = os.path.join(data_home, "ijcnn1.t")
    test_file = os.path.join(data_home, "ijcnn1.tr")
    return _todense(_load(train_file, test_file, "ijcnn"))


def load_mnist8():
    X_train, y_train, X_test, y_test = load_mnist()
    selected = y_train == 8
    y_train[selected] = 1
    y_train[~selected] = 0
    selected = y_test == 8
    y_test[selected] = 1
    y_test[~selected] = 0
    return X_train, y_train, X_test, y_test


def load_realsim():
    data_home = get_data_home()
    train_file = os.path.join(data_home, "realsim")
    return _load(train_file, None, "realsim")


def load_reuters():
    data_home = get_data_home()
    train_file = os.path.join(data_home, "reuters", "money-fx.trn")
    test_file = os.path.join(data_home, "reuters", "money-fx.tst")
    return _load(train_file, test_file, "reuters")


def load_protein0():
    X_train, y_train, X_test, y_test = load_protein()
    selected = y_train == 0
    y_train[selected] = 1
    y_train[~selected] = 0
    selected = y_test == 0
    y_test[selected] = 1
    y_test[~selected] = 0
    return X_train, y_train, X_test, y_test


def load_usps0():
    X_train, y_train, X_test, y_test = load_usps()
    selected = y_train == 10
    y_train[selected] = 1
    y_train[~selected] = 0
    selected = y_test == 10
    y_test[selected] = 1
    y_test[~selected] = 0
    return X_train, y_train, X_test, y_test


def load_usps0_noisy():
    X_train, y_train, X_test, y_test = load_usps0()
    n_samples = X_train.shape[0]
    indices = np.arange(n_samples)
    random_state = check_random_state(0)
    random_state.shuffle(indices)
    n = n_samples / 10
    indices = indices[:n]
    y_train[indices] = np.logical_not(y_train[indices]).astype(int)
    return X_train, y_train, X_test, y_test


def load_url():
    data_home = get_data_home()
    train_file = os.path.join(data_home, "url_combined")
    return _load(train_file, None, "url")


def load_waveform():
    data_home = get_data_home()
    train_file = os.path.join(data_home, "waveform", "waveform.all.txt")
    return _todense(_load(train_file, None, "waveform"))


def load_webspam():
    data_home = get_data_home()
    train_file = os.path.join(data_home, "webspam")
    return _load(train_file, None, "webspam")


# multi-class

def load_dna():
    data_home = get_data_home()
    train_file = os.path.join(data_home, "dna.scale.tr")
    test_file = os.path.join(data_home, "dna.scale.t")
    return _todense(_load(train_file, test_file, "dna"))


def load_letter():
    data_home = get_data_home()
    train_file = os.path.join(data_home, "letter.scale.tr")
    test_file = os.path.join(data_home, "letter.scale.t")
    return _todense(_load(train_file, test_file, "letter"))


def load_mnist():
    data_home = get_data_home()
    train_file = os.path.join(data_home, "mnist.scale")
    test_file = os.path.join(data_home, "mnist.scale.t")
    return _load(train_file, test_file, "mnist")


def load_news20():
    data_home = get_data_home()
    train_file = os.path.join(data_home, "news20.scale")
    test_file = os.path.join(data_home, "news20.t.scale")
    return _load(train_file, test_file, "news20")


def load_pendigits():
    data_home = get_data_home()
    train_file = os.path.join(data_home, "pendigits")
    test_file = os.path.join(data_home, "pendigits.t")
    return _todense(_load(train_file, test_file, "pendigits"))


def load_protein():
    data_home = get_data_home()
    train_file = os.path.join(data_home, "protein.tr")
    test_file = os.path.join(data_home, "protein.t")
    return _todense(_load(train_file, test_file, "protein"))


def load_rcv1():
    data_home = get_data_home()
    train_file = os.path.join(data_home, "rcv1_train.multiclass")
    test_file = os.path.join(data_home, "rcv1_test.multiclass")
    return _load(train_file, test_file, "rcv1")


def load_satimage():
    data_home = get_data_home()
    train_file = os.path.join(data_home, "satimage.scale.tr")
    test_file = os.path.join(data_home, "satimage.scale.t")
    return _todense(_load(train_file, test_file, "satimage"))


def load_sector():
    data_home = get_data_home()
    train_file = os.path.join(data_home, "sector.scale")
    test_file = os.path.join(data_home, "sector.t.scale")
    return _load(train_file, test_file, "sector")


def load_usps():
    data_home = get_data_home()
    train_file = os.path.join(data_home, "usps")
    test_file = os.path.join(data_home, "usps.t")
    return _todense(_load(train_file, test_file, "usps"))


def load_usps_noisy():
    X_train, y_train, X_test, y_test = load_usps()
    n_samples = X_train.shape[0]
    n = n_samples / 10
    random_state = check_random_state(0)
    indices = np.arange(n_samples)
    random_state.shuffle(indices)
    indices2 = np.arange(n_samples)
    random_state.shuffle(indices2)
    y_train[indices[:n]] = y_train[indices2[:n]]
    return X_train, y_train, X_test, y_test



LOADERS = {
            # regression
            "abalone": load_abalone,
            "cpusmall": load_cpusmall,
            "cadata": load_cadata,
            "space_ga": load_space_ga,
            "YearPredictionMSD": load_YearPredictionMSD,
            # binary classification
            "adult": load_adult,
            "banana": load_banana,
            "covtype": load_covtype,
            "covtype_subset": load_covtype_subset,
            "ijcnn": load_ijcnn,
            "mnist8": load_mnist8,
            "realsim": load_realsim,
            "reuters": load_reuters,
            "protein0": load_protein0,
            "url" : load_url,
            "usps0": load_usps0,
            "usps0_noisy": load_usps0_noisy,
            "waveform": load_waveform,
            "webspam": load_webspam,
            # multi-class
            "dna": load_dna,
            "letter": load_letter,
            "news20" : load_news20,
            "mnist": load_mnist,
            "satimage": load_satimage,
            "sector": load_sector,
            "pendigits": load_pendigits,
            "protein": load_protein,
            "rcv1": load_rcv1,
            "usps": load_usps,
            "usps_noisy": load_usps_noisy,
}


def get_loader(dataset):
    return LOADERS[dataset]


def load_dataset(dataset, group_all=False):
    ret = get_loader(dataset)()

    if group_all and len(ret) > 2 and ret[2] is not None:
        X_tr, y_tr, X_te, y_te = ret

        if hasattr(X_tr, "tocsr"):
            data = sp.vstack((X_tr, X_te)).tocsr()
        else:
            data = np.vstack((X_tr, X_te))
        target = np.concatenate((y_tr, y_te))

        return (data, target)
    else:
        return ret[:2]



"""
==========================================
Primal (Block) Coordinate Descent Solvers
==========================================

This module provides (block) coordinate descent solvers for a variety of loss
functions and penalties.
"""

# Author: Mathieu Blondel
# License: BSD

import numpy as np

from joblib import Parallel, delayed
from six.moves import xrange

from .base import BaseClassifier
from .base import BaseRegressor

from .dataset_fast import get_dataset
from .primal_cd_fast import _primal_cd

from .primal_cd_fast import Squared
from .primal_cd_fast import SmoothHinge
from .primal_cd_fast import SquaredHinge
from .primal_cd_fast import ModifiedHuber
from .primal_cd_fast import Log


class _BaseCD(object):

    def _get_loss(self):
        params = {"max_steps": self._get_max_steps(),
                  "sigma": self.sigma,
                  "beta": self.beta,
                  "verbose": self.verbose}

        losses = {
            "squared": Squared(verbose=self.verbose),
            "smooth_hinge": SmoothHinge(**params),
            "squared_hinge": SquaredHinge(**params),
            "modified_huber": ModifiedHuber(**params),
            "log": Log(**params),
        }

        return losses[self.loss]

    def _get_max_steps(self):
        if self.max_steps == "auto":
            if self.loss == "log":
                max_steps = 0
            else:
                max_steps = 30
        else:
            max_steps = self.max_steps
        return max_steps

    def _get_penalty(self):
        penalties = {
            "l1": 1,
            "l2": 2,
        }
        return penalties[self.penalty]

    def _init_errors(self, Y):
        n_samples, n_vectors = Y.shape
        if self.loss == "squared":
            self.errors_ = -Y.T
        else:
            self.errors_ = np.ones((n_vectors, n_samples), dtype=np.float64)


class CDClassifier(_BaseCD, BaseClassifier):
    """Estimator for learning linear classifiers by (block) coordinate descent.

    The objective functions considered take the form

    minimize F(W) = C * L(W) + alpha * R(W),

    where L(W) is a loss term and R(W) is a penalty term.

    Parameters
    ----------
    loss : str, 'squared_hinge', 'log', 'modified_huber', 'squared'
        The loss function to be used.

    penalty : str, 'l2', 'l1', 'l1/l2'
        The penalty to be used.

        - l2: ridge
        - l1: lasso
        - l1/l2: group lasso

    multiclass : bool
        Whether to use a direct multiclass formulation (True) or one-vs-rest
        (False). Direct formulations are only available for loss='squared_hinge'
        and loss='log'.

    C : float
        Weight of the loss term.

    alpha : float
        Weight of the penalty term.

    max_iter : int
        Maximum number of iterations to perform.

    tol : float
        Tolerance of the stopping criterion.

    termination : str, 'violation_sum', 'violation_max'
        Stopping criterion to use.

    shrinking : bool
        Whether to activate shrinking or not.

    max_steps : int or "auto"
        Maximum number of steps to use during the line search. Use max_steps=0
        to use a constant step size instead of the line search. Use
        max_steps="auto" to let CDClassifier choose the best value.

    sigma : float
        Constant used in the line search sufficient decrease condition.

    beta : float
        Multiplicative constant used in the backtracking line search.

    warm_start : bool
        Whether to activate warm-start or not.

    debiasing : bool
        Whether to refit the model using l2 penalty (only useful if penalty='l1'
        or penalty='l1/l2').

    Cd : float
        Value of `C` when doing debiasing.

    warm_debiasing : bool
        Whether to warm-start the model or not when doing debiasing.

    selection : str, 'cyclic', 'uniform'
        Strategy to use for selecting coordinates.

    permute : bool
        Whether to permute coordinates or not before cycling (only when
        selection='cyclic').

    callback : callable
        Callback function.

    n_calls : int
        Frequency with which `callback` must be called.

    random_state : RandomState or int
        The seed of the pseudo random number generator to use.

    verbose : int
        Verbosity level.

    n_jobs : int
        Number of CPU's to be used when `multiclass=False` and when
        penalty is a non group-lasso penalty. By default use one CPU.
        If set to -1, use all CPU's

    Example
    -------

    The following example demonstrates how to learn a classification
    model with a multiclass squared hinge loss and an l1/l2 penalty.

    >>> from sklearn.datasets import fetch_20newsgroups_vectorized
    >>> from lightning.classification import CDClassifier
    >>> bunch = fetch_20newsgroups_vectorized(subset="all")
    >>> X, y = bunch.data, bunch.target
    >>> clf = CDClassifier(penalty="l1/l2",
                           loss="squared_hinge",
                           multiclass=True,
                           max_iter=20,
                           alpha=1e-4,
                           C=1.0 / X.shape[0],
                           tol=1e-3,
                           random_state=0).fit(X, y)
    >>> accuracy = clf.score(X, y)

    References
    ----------
    Block Coordinate Descent Algorithms for Large-scale Sparse Multiclass
    Classification.  Mathieu Blondel, Kazuhiro Seki, and Kuniaki Uehara.
    Machine Learning, May 2013.
    """

    def __init__(self, loss="squared_hinge", penalty="l2", multiclass=False,
                 C=1.0, alpha=1.0,
                 max_iter=50, tol=1e-3, termination="violation_sum",
                 shrinking=True,
                 max_steps="auto", sigma=0.01, beta=0.5,
                 warm_start=False, debiasing=False, Cd=1.0,
                 warm_debiasing=False,
                 selection="cyclic", permute=True,
                 callback=None, n_calls=100,
                 random_state=None, verbose=0, n_jobs=1):
        self.C = C
        self.alpha = alpha
        self.loss = loss
        self.penalty = penalty
        self.multiclass = multiclass
        self.max_iter = max_iter
        self.tol = tol
        self.termination = termination
        self.shrinking = shrinking
        self.max_steps = max_steps
        self.sigma = sigma
        self.beta = beta
        self.warm_start = warm_start
        self.debiasing = debiasing
        self.Cd = Cd
        self.warm_debiasing = warm_debiasing
        self.selection = selection
        self.permute = permute
        self.callback = callback
        self.n_calls = n_calls
        self.random_state = random_state
        self.verbose = verbose
        self.coef_ = None
        self.violation_init_ = {}
        self.n_jobs = n_jobs

    def fit(self, X, y):
        """Fit model according to X and y.

        Parameters
        ----------
        X : array-like, shape = [n_samples, n_features]
            Training vectors, where n_samples is the number of samples
            and n_features is the number of features.

        y : array-like, shape = [n_samples]
            Target values.

        Returns
        -------
        self : classifier
            Returns self.
        """
        rs = self._get_random_state()

        # Create dataset
        ds = get_dataset(X, order="fortran")
        n_samples = ds.get_n_samples()
        n_features = ds.get_n_features()

        if self.penalty != "l1/l2" and self.multiclass:
            raise NotImplementedError("True multiclass options not implemented "
                                      "for non group-lasso(l1/l2) penalties.")

        # Create label transformers
        #neg_label = 0 if self.penalty == "nn" else -1
        reencode = self.penalty == "l1/l2"
        y, n_classes, n_vectors = self._set_label_transformers(y,
                                                               reencode,
                                                               neg_label=-1)
        Y = np.asfortranarray(self.label_binarizer_.transform(y),
                              dtype=np.float64)

        # Initialize coefficients
        if not self.warm_start or self.coef_ is None:
            self.C_init = self.C
            self.coef_ = np.zeros((n_vectors, n_features), dtype=np.float64)
            self._init_errors(Y)

        self.intercept_ = np.zeros(n_vectors, dtype=np.float64)
        indices = np.arange(n_features, dtype=np.int32)

        max_steps = self._get_max_steps()

        # Learning
        if self.penalty == "l1/l2":
            tol = self.tol
            #n_min = np.min(np.sum(Y == 1, axis=0))
            #tol *= max(n_min, 1) / n_samples

            vinit = self.violation_init_.get(0, 0) * self.C / self.C_init
            model = _primal_cd(self, self.coef_, self.errors_,
                              ds, y, Y, -1, self.multiclass,
                              indices, 12, self._get_loss(),
                              self.selection, self.permute, self.termination,
                              self.C, self.alpha,
                              self.max_iter, max_steps,
                              self.shrinking, vinit,
                              rs, tol, self.callback, self.n_calls,
                              self.verbose)
            viol = model[0]
            if self.warm_start and len(self.violation_init_) == 0:
                self.violation_init_[0] = viol

        elif self.penalty in ("l1", "l2", "nn"):
            penalty = self._get_penalty()

            n_pos = np.zeros(n_vectors)
            vinit = self.C / self.C_init * np.ones_like(n_pos)
            for k in xrange(n_vectors):
                n_pos[k] = np.sum(Y[:, k] == 1)
                vinit[k] *= self.violation_init_.get(k, 0)
            n_neg = n_samples - n_pos
            tol = self.tol * np.maximum(np.minimum(n_pos, n_neg), 1) / n_samples

            jobs = (delayed(_primal_cd)(self, self.coef_, self.errors_,
                                        ds, y, Y, k, False,
                                        indices, penalty, self._get_loss(),
                                        self.selection, self.permute,
                                        self.termination,
                                        self.C, self.alpha,
                                        self.max_iter, max_steps,
                                        self.shrinking, vinit[k],
                                        rs, tol[k], self.callback, self.n_calls,
                                        self.verbose)
                    for k in xrange(n_vectors))
            model = Parallel(n_jobs=self.n_jobs, verbose=self.verbose)(jobs)
            viol, coefs, errors = zip(*model)
            self.coef_ = np.asarray(coefs)
            self.errors_ = np.asarray(errors)

            for k in range(n_vectors):
                if self.warm_start and not k in self.violation_init_:
                    self.violation_init_[k] = viol[k]

        if self.debiasing:
            nz = self.coef_ != 0

            if not self.warm_debiasing:
                self.coef_ = np.zeros((n_vectors, n_features), dtype=np.float64)
                self._init_errors(Y)

            indices = np.arange(n_features, dtype=np.int32)
            jobs = (delayed(_primal_cd)(
                           self, self.coef_, self.errors_,
                           ds, y, Y, k, False,
                           indices[nz[k]], 2, self._get_loss(),
                           "cyclic", self.permute,
                           "violation_sum",
                           self.Cd, 1.0,
                           self.max_iter, max_steps,
                           False, 0,
                           rs, self.tol, self.callback, self.n_calls,
                           self.verbose
                            )
                    for k in xrange(n_vectors))
            model = Parallel(n_jobs=self.n_jobs, verbose=self.verbose)(jobs)
            viol, coefs, errors = zip(*model)
            self.coef_ = np.asarray(coefs)
            self.errors_ = np.asarray(errors)

        return self


class CDRegressor(_BaseCD, BaseRegressor):
    """Estimator for learning linear regressors by (block) coordinate descent.

    The objective functions considered take the form

    minimize F(W) = C * L(W) + alpha * R(W),

    where L(W) is a loss term and R(W) is a penalty term.

    Parameters
    ----------
    loss : str, 'squared'
        The loss function to be used.

    penalty : str, 'l2', 'l1', 'l1/l2', 'nnl1', 'nnl2'
        The penalty to be used.

        - l2: ridge
        - l1: lasso
        - l1/l2: group lasso
        - nnl1: non-negative constraints + l1 penalty
        - nnl2: non-negative constraints + l2 penalty

    For other parameters, see `CDClassifier`.
    """

    def __init__(self, C=1.0, alpha=1.0,
                 loss="squared", penalty="l2",
                 max_iter=50, tol=1e-3, termination="violation_sum",
                 shrinking=True,
                 max_steps=30, sigma=0.01, beta=0.5,
                 warm_start=False, debiasing=False, Cd=1.0,
                 warm_debiasing=False,
                 selection="cyclic", permute=True,
                 callback=None, n_calls=100,
                 random_state=None, verbose=0, n_jobs=1):
        self.C = C
        self.alpha = alpha
        self.loss = loss
        self.penalty = penalty
        self.max_iter = max_iter
        self.tol = tol
        self.termination = termination
        self.shrinking = shrinking
        self.max_steps = max_steps
        self.sigma = sigma
        self.beta = beta
        self.warm_start = warm_start
        self.debiasing = debiasing
        self.Cd = Cd
        self.warm_debiasing = warm_debiasing
        self.selection = selection
        self.permute = permute
        self.callback = callback
        self.n_calls = n_calls
        self.random_state = random_state
        self.verbose = verbose
        self.coef_ = None
        self.violation_init_ = {}
        self.n_jobs = n_jobs

    def fit(self, X, y):
        """Fit model according to X and y.

        Parameters
        ----------
        X : array-like, shape = [n_samples, n_features]
            Training vectors, where n_samples is the number of samples
            and n_features is the number of features.

        y : array-like, shape = [n_samples] or [n_samples, n_targets]
            Target values.

        Returns
        -------
        self : regressor
            Returns self.
        """
        rs = self._get_random_state()

        # Create dataset
        ds = get_dataset(X, order="fortran")
        n_features = ds.get_n_features()

        self.outputs_2d_ = len(y.shape) == 2
        if self.outputs_2d_:
            Y = y
        else:
            Y = y.reshape(-1, 1)
        Y = np.asfortranarray(Y, dtype=np.float64)
        y = np.empty(0, dtype=np.int32)
        n_vectors = Y.shape[1]

        # Initialize coefficients
        if not self.warm_start or self.coef_ is None:
            self.C_init = self.C
            self.coef_ = np.zeros((n_vectors, n_features), dtype=np.float64)
            self._init_errors(Y)

        self.intercept_ = np.zeros(n_vectors, dtype=np.float64)
        indices = np.arange(n_features, dtype=np.int32)


        if self.penalty == "l1/l2":
            vinit = self.violation_init_.get(0, 0) * self.C / self.C_init
            model = _primal_cd(self, self.coef_, self.errors_,
                              ds, y, Y, -1, False,
                              indices, 12, self._get_loss(),
                              self.selection, self.permute, self.termination,
                              self.C, self.alpha,
                              self.max_iter, self.max_steps,
                              self.shrinking, vinit,
                              rs, self.tol, self.callback, self.n_calls,
                              self.verbose)
            viol = model[0]
            if self.warm_start and len(self.violation_init_) == 0:
                self.violation_init_[0] = viol
        else:
            penalty = self._get_penalty()
            vinit = np.asarray([self.violation_init_.get(k, 0)
                    for k in xrange(n_vectors)]) * self.C / self.C_init

            jobs = (delayed(_primal_cd)(self, self.coef_, self.errors_,
                                       ds, y, Y, k, False,
                                       indices, penalty, self._get_loss(),
                                       self.selection, self.permute,
                                       self.termination,
                                       self.C, self.alpha,
                                       self.max_iter, self.max_steps,
                                       self.shrinking, vinit[k],
                                       rs, self.tol, self.callback, self.n_calls,
                                       self.verbose)
                    for k in xrange(n_vectors))

            model = Parallel(n_jobs=self.n_jobs, verbose=self.verbose)(jobs)
            viol, self.coef_, self.error_ = zip(*model)
            self.coef_ = np.asarray(self.coef_)
            self.error_ = np.asarray(self.error_)

            if self.warm_start and not n_vectors in self.violation_init_:
                self.violation_init_[n_vectors] = viol

        return self



#  tests for sklearn-contrib-lightning-0.6.0-py36he38d939_0 (this is a generated file);
print('===== testing package: sklearn-contrib-lightning-0.6.0-py36he38d939_0 =====');
print('running run_test.py');
#  --- run_test.py (begin) ---
import lightning
import nose
import os

lightning_dir = os.path.dirname(
    os.path.abspath(lightning.__file__))
os.chdir(lightning_dir)
nose.run(module=lightning)
#  --- run_test.py (end) ---

print('===== sklearn-contrib-lightning-0.6.0-py36he38d939_0 OK =====');
print("import: 'lightning'")
import lightning




# -*- coding: utf-8 -*-
from sklearn.utils.testing import assert_array_equal, assert_equal


def check_predict_proba(clf, X):
    y_pred = clf.predict(X)
    n_samples = y_pred.shape[0]
    # normalize negative class to 0 (it is sometimes 0, sometimes -1)
    y_pred = (y_pred == 1)

    # check that predict_proba result agree with y_true
    y_proba = clf.predict_proba(X)
    assert_equal(y_proba.shape, (n_samples, 2))
    y_proba_best = (y_proba.argmax(axis=1) == 1)
    assert_array_equal(y_proba_best, y_pred)

    # check that y_proba looks like probability
    assert not (y_proba > 1).any()
    assert not (y_proba < 0).any()
    assert_array_equal(y_proba.sum(axis=1), 1.0)



import numpy as np

from sklearn.datasets import load_iris
from sklearn.utils.testing import assert_equal
from sklearn.utils.testing import assert_almost_equal

from lightning.classification import SDCAClassifier
from lightning.regression import SDCARegressor
from lightning.impl.tests.utils import check_predict_proba


iris = load_iris()
X, y = iris.data, iris.target

X_bin = X[y <= 1]
y_bin = y[y <= 1] * 2 - 1


def test_sdca_hinge():
    clf = SDCAClassifier(loss="hinge", random_state=0)
    clf.fit(X_bin, y_bin)
    assert not hasattr(clf, 'predict_proba')
    assert_equal(clf.score(X_bin, y_bin), 1.0)


def test_sdca_hinge_multiclass():
    clf = SDCAClassifier(alpha=1e-2, max_iter=100, loss="hinge",
                              random_state=0)
    clf.fit(X, y)
    assert_almost_equal(clf.score(X, y), 0.933, 3)


def test_sdca_squared():
    clf = SDCAClassifier(loss="squared", random_state=0)
    clf.fit(X_bin, y_bin)
    assert not hasattr(clf, 'predict_proba')
    assert_equal(clf.score(X_bin, y_bin), 1.0)


def test_sdca_absolute():
    clf = SDCAClassifier(loss="absolute", random_state=0)
    clf.fit(X_bin, y_bin)
    assert not hasattr(clf, 'predict_proba')
    assert_equal(clf.score(X_bin, y_bin), 1.0)


def test_sdca_hinge_elastic():
    clf = SDCAClassifier(alpha=0.5, l1_ratio=0.85, loss="hinge",
                              random_state=0)
    clf.fit(X_bin, y_bin)
    assert_equal(clf.score(X_bin, y_bin), 1.0)


def test_sdca_smooth_hinge_elastic():
    clf = SDCAClassifier(alpha=0.5, l1_ratio=0.85, loss="smooth_hinge",
                              random_state=0)
    clf.fit(X_bin, y_bin)
    assert not hasattr(clf, 'predict_proba')
    assert_equal(clf.score(X_bin, y_bin), 1.0)


def test_sdca_squared_hinge_elastic():
    clf = SDCAClassifier(alpha=0.5, l1_ratio=0.85, loss="squared_hinge",
                              random_state=0)
    clf.fit(X_bin, y_bin)
    assert_equal(clf.score(X_bin, y_bin), 1.0)


def test_sdca_hinge_l1_only():
    clf = SDCAClassifier(alpha=0.5, l1_ratio=1.0, loss="hinge", tol=1e-2,
                              max_iter=200, random_state=0)
    clf.fit(X_bin, y_bin)
    assert_equal(clf.score(X_bin, y_bin), 1.0)


def test_sdca_smooth_hinge_l1_only():
    clf = SDCAClassifier(alpha=0.5, l1_ratio=1.0, loss="smooth_hinge",
                              tol=1e-2, max_iter=200, random_state=0)
    clf.fit(X_bin, y_bin)
    assert_equal(clf.score(X_bin, y_bin), 1.0)


def test_sdca_squared_l1_only():
    clf = SDCAClassifier(alpha=0.5, l1_ratio=1.0, loss="squared", tol=1e-2,
                              max_iter=100, random_state=0)
    clf.fit(X_bin, y_bin)
    assert_equal(clf.score(X_bin, y_bin), 1.0)


def test_sdca_absolute_l1_only():
    clf = SDCAClassifier(alpha=0.5, l1_ratio=1.0, loss="absolute",
                              tol=1e-2, max_iter=200, random_state=0)
    clf.fit(X_bin, y_bin)
    assert_equal(clf.score(X_bin, y_bin), 1.0)


def test_sdca_callback():
    class Callback(object):

        def __init__(self, X, y):
            self.X = X
            self.y = y
            self.acc = []

        def __call__(self, clf):
            score = clf.score(self.X, self.y)
            self.acc.append(score)

    cb = Callback(X_bin, y_bin)
    clf = SDCAClassifier(alpha=0.5, l1_ratio=0.85, loss="hinge",
                              callback=cb, random_state=0)
    clf.fit(X_bin, y_bin)
    assert_equal(cb.acc[0], 0.5)
    assert_equal(cb.acc[-1], 1.0)


def test_bin_classes():
    clf = SDCAClassifier()
    clf.fit(X_bin, y_bin)
    assert_equal(list(clf.classes_), [-1, 1])


def test_multiclass_classes():
    clf = SDCAClassifier()
    clf.fit(X, y)
    assert_equal(list(clf.classes_), [0, 1, 2])


def test_sdca_regression():
    for loss in ("squared", "absolute"):
        reg = SDCARegressor(loss=loss)
        reg.fit(X_bin, y_bin)
        y_pred = np.sign(reg.predict(X_bin))
        assert_equal(np.mean(y_bin == y_pred), 1.0)




import numpy as np
import scipy.sparse as sp

from scipy.linalg import svd, diagsvd

from sklearn.utils.testing import assert_almost_equal
from sklearn.utils.testing import assert_true
from sklearn.utils.testing import assert_equal

from sklearn.datasets import load_digits

from lightning.impl.datasets.samples_generator import make_classification
from lightning.classification import FistaClassifier
from lightning.regression import FistaRegressor
from lightning.impl.penalty import project_simplex, project_l1_ball, L1Penalty

bin_dense, bin_target = make_classification(n_samples=200, n_features=100,
                                            n_informative=5,
                                            n_classes=2, random_state=0)
bin_target = bin_target * 2 - 1

mult_dense, mult_target = make_classification(n_samples=300, n_features=100,
                                              n_informative=5,
                                              n_classes=3, random_state=0)
bin_csr = sp.csr_matrix(bin_dense)
mult_csr = sp.csr_matrix(mult_dense)
digit = load_digits(2)


def test_fista_multiclass_l1l2():
    for data in (mult_dense, mult_csr):
        clf = FistaClassifier(max_iter=200, penalty="l1/l2", multiclass=True)
        clf.fit(data, mult_target)
        assert_almost_equal(clf.score(data, mult_target), 0.99, 2)


def test_fista_multiclass_l1l2_log():
    for data in (mult_dense, mult_csr):
        clf = FistaClassifier(max_iter=200, penalty="l1/l2", loss="log",
                              multiclass=True)
        clf.fit(data, mult_target)
        assert_almost_equal(clf.score(data, mult_target), 0.90, 2)

def test_fista_multiclass_l1l2_log_margin():
    for data in (mult_dense, mult_csr):
        clf = FistaClassifier(max_iter=200, penalty="l1/l2", loss="log_margin",
                              multiclass=True)
        clf.fit(data, mult_target)
        assert_almost_equal(clf.score(data, mult_target), 0.93, 2)


def test_fista_multiclass_l1():
    for data in (mult_dense, mult_csr):
        clf = FistaClassifier(max_iter=200, penalty="l1", multiclass=True)
        clf.fit(data, mult_target)
        assert_almost_equal(clf.score(data, mult_target), 0.98, 2)



def test_fista_multiclass_tv1d():
    for data in (mult_dense, mult_csr):
        clf = FistaClassifier(max_iter=200, penalty="tv1d", multiclass=True)
        clf.fit(data, mult_target)
        assert_almost_equal(clf.score(data, mult_target), 0.97, 2)

        # adding a lot of regularization coef_ should be constant
        clf = FistaClassifier(max_iter=200, penalty="tv1d", multiclass=True, alpha=1e6)
        clf.fit(data, mult_target)
        for i in range(clf.coef_.shape[0]):
            np.testing.assert_array_almost_equal(
                clf.coef_[i], np.mean(clf.coef_[i]) * np.ones(data.shape[1]))


def test_fista_multiclass_l1l2_no_line_search():
    for data in (mult_dense, mult_csr):
        clf = FistaClassifier(max_iter=500, penalty="l1/l2", multiclass=True,
                              max_steps=0)
        clf.fit(data, mult_target)
        assert_almost_equal(clf.score(data, mult_target), 0.94, 2)


def test_fista_multiclass_l1_no_line_search():
    for data in (mult_dense, mult_csr):
        clf = FistaClassifier(max_iter=500, penalty="l1", multiclass=True,
                              max_steps=0)
        clf.fit(data, mult_target)
        assert_almost_equal(clf.score(data, mult_target), 0.94, 2)


def test_fista_bin_l1():
    for data in (bin_dense, bin_csr):
        clf = FistaClassifier(max_iter=200, penalty="l1")
        clf.fit(data, bin_target)
        assert_almost_equal(clf.score(data, bin_target), 1.0, 2)


def test_fista_bin_l1_no_line_search():
    for data in (bin_dense, bin_csr):
        clf = FistaClassifier(max_iter=500, penalty="l1", max_steps=0)
        clf.fit(data, bin_target)
        assert_almost_equal(clf.score(data, bin_target), 1.0, 2)


def test_fista_multiclass_trace():
    for data in (mult_dense, mult_csr):
        clf = FistaClassifier(max_iter=100, penalty="trace", multiclass=True)
        clf.fit(data, mult_target)
        assert_almost_equal(clf.score(data, mult_target), 0.96, 2)


def test_fista_bin_classes():
    clf = FistaClassifier()
    clf.fit(bin_dense, bin_target)
    assert_equal(list(clf.classes_), [0, 1])


def test_fista_multiclass_classes():
    clf = FistaClassifier()
    clf.fit(mult_dense, mult_target)
    assert_equal(list(clf.classes_), [0, 1, 2])


def test_fista_regression():
    reg = FistaRegressor(max_iter=100, verbose=0)
    reg.fit(bin_dense, bin_target)
    y_pred = np.sign(reg.predict(bin_dense))
    assert_almost_equal(np.mean(bin_target == y_pred), 0.985)


def test_fista_regression_simplex():
    rng = np.random.RandomState(0)
    w = project_simplex(rng.rand(10))
    X = rng.randn(1000, 10)
    y = np.dot(X, w)

    reg = FistaRegressor(penalty="simplex", max_iter=100, verbose=0)
    reg.fit(X, y)
    y_pred = reg.predict(X)
    error = np.sqrt(np.mean((y - y_pred) ** 2))
    assert_almost_equal(error, 0.000, 3)
    assert_true(np.all(reg.coef_ >= 0))
    assert_almost_equal(np.sum(reg.coef_), 1.0, 3)


def test_fista_regression_l1_ball():
    rng = np.random.RandomState(0)
    alpha = 5.0
    w = project_simplex(rng.randn(10), alpha)
    X = rng.randn(1000, 10)
    y = np.dot(X, w)

    reg = FistaRegressor(penalty="l1-ball", alpha=alpha, max_iter=100, verbose=0)
    reg.fit(X, y)
    y_pred = reg.predict(X)
    error = np.sqrt(np.mean((y - y_pred) ** 2))
    assert_almost_equal(error, 0.000, 3)
    assert_almost_equal(np.sum(np.abs(reg.coef_)), alpha, 3)


def test_fista_regression_trace():
    rng = np.random.RandomState(0)
    def _make_data(n_samples, n_features, n_tasks, n_components):
        W = rng.rand(n_tasks, n_features) - 0.5
        U, S, V = svd(W, full_matrices=True)
        S[n_components:] = 0
        S = diagsvd(S, U.shape[0], V.shape[0])
        W = np.dot(np.dot(U, S), V)
        X = rng.rand(n_samples, n_features) - 0.5
        Y = np.dot(X, W.T)
        return X, Y, W

    X, Y, W = _make_data(200, 50,30, 5)
    reg = FistaRegressor(max_iter=15, verbose=0)
    reg.fit(X, Y)
    Y_pred = reg.predict(X)
    error = (Y_pred - Y).ravel()
    error = np.dot(error, error)
    assert_almost_equal(error, 77.44, 2)


def test_fista_custom_prox():
    # test FISTA with a custom prox
    l1_pen = L1Penalty()
    for data in (bin_dense, bin_csr):
        clf = FistaClassifier(max_iter=500, penalty="l1", max_steps=0)
        clf.fit(data, bin_target)

        clf2 = FistaClassifier(max_iter=500, penalty=l1_pen, max_steps=0)
        clf2.fit(data, bin_target)
        np.testing.assert_array_almost_equal_nulp(clf.coef_.ravel(), clf2.coef_.ravel())



import os.path

import numpy


def configuration(parent_package='', top_path=None):
    from numpy.distutils.misc_util import Configuration

    config = Configuration('impl', parent_package, top_path)

    randomdir = os.path.join(top_path, "lightning", "impl", "randomkit")

    config.add_extension('adagrad_fast',
                         sources=['adagrad_fast.pyx'],
                         language='c++',
                         include_dirs=[numpy.get_include(), randomdir])

    config.add_extension('dataset_fast',
                         sources=['dataset_fast.pyx'],
                         language='c++',
                         include_dirs=[numpy.get_include(), randomdir])

    config.add_extension('dual_cd_fast',
                         sources=['dual_cd_fast.pyx'],
                         language='c++',
                         include_dirs=[numpy.get_include(), randomdir])

    config.add_extension('loss_fast',
                         sources=['loss_fast.pyx'],
                         language='c++',
                         include_dirs=[numpy.get_include(), randomdir])

    config.add_extension('prank_fast',
                         sources=['prank_fast.pyx'],
                         language='c++',
                         include_dirs=[numpy.get_include(), randomdir])

    config.add_extension('primal_cd_fast',
                         sources=['primal_cd_fast.pyx'],
                         language='c++',
                         include_dirs=[numpy.get_include(), randomdir])

    config.add_extension('prox_fast',
                         sources=['prox_fast.pyx'],
                         language='c++',
                         include_dirs=[numpy.get_include(), randomdir])

    config.add_extension('sag_fast',
                         sources=['sag_fast.pyx'],
                         language='c++',
                         include_dirs=[numpy.get_include(), randomdir])

    config.add_extension('sdca_fast',
                         sources=['sdca_fast.pyx'],
                         language='c++',
                         include_dirs=[numpy.get_include(), randomdir])

    config.add_extension('sgd_fast',
                         sources=['sgd_fast.pyx'],
                         language='c++',
                         include_dirs=[numpy.get_include(), randomdir])

    config.add_extension('svrg_fast',
                         sources=['svrg_fast.pyx'],
                         language='c++',
                         include_dirs=[numpy.get_include(), randomdir])

    config.add_subpackage('datasets')
    config.add_subpackage('randomkit')
    config.add_subpackage('tests')

    # add .pxd files to be re-used by third party software
    config.add_data_files('sag_fast.pxd', 'dataset_fast.pxd',
                          'sgd_fast.pxd', 'prox_fast.pxd')

    return config

if __name__ == '__main__':
    from numpy.distutils.core import setup
    setup(**configuration(top_path='').todict())



# Author: Mathieu Blondel
# License: BSD

cdef class Dataset:

    cdef int n_samples
    cdef int n_features

    cpdef int get_n_samples(self)
    cpdef int get_n_features(self)


cdef class RowDataset(Dataset):

    cdef void get_row_ptr(self,
                             int i,
                             int** indices,
                             double** data,
                             int* n_nz) nogil

    cpdef get_row(self, int i)


cdef class ColumnDataset(Dataset):

    cdef void get_column_ptr(self,
                             int j,
                             int** indices,
                             double** data,
                             int* n_nz) nogil

    cpdef get_column(self, int j)


cdef class ContiguousDataset(RowDataset):

    cdef int* indices
    cdef double* data
    cdef object X

    cdef void get_row_ptr(self,
                          int i,
                          int** indices,
                          double** data,
                          int* n_nz) nogil


cdef class FortranDataset(ColumnDataset):

    cdef int* indices
    cdef double* data
    cdef object X

    cdef void get_column_ptr(self,
                             int j,
                             int** indices,
                             double** data,
                             int* n_nz) nogil


cdef class CSRDataset(RowDataset):

    cdef int* indices
    cdef double* data
    cdef int* indptr
    cdef object X

    cdef void get_row_ptr(self,
                          int i,
                          int** indices,
                          double** data,
                          int* n_nz) nogil


cdef class CSCDataset(ColumnDataset):

    cdef int* indices
    cdef double* data
    cdef int* indptr
    cdef object X

    cdef void get_column_ptr(self,
                             int j,
                             int** indices,
                             double** data,
                             int* n_nz) nogil



from sklearn.utils.testing import assert_almost_equal
from sklearn.utils.testing import assert_equal

from lightning.impl.datasets.samples_generator import make_classification
from lightning.impl.primal_newton import KernelSVC

bin_dense, bin_target = make_classification(n_samples=200, n_features=100,
                                            n_informative=5,
                                            n_classes=2, random_state=0)

def test_kernel_svc():
    clf = KernelSVC(kernel="rbf", gamma=0.1, random_state=0, verbose=0)
    clf.fit(bin_dense, bin_target)
    assert_almost_equal(clf.score(bin_dense, bin_target), 1.0)
    assert_equal(list(clf.classes_), [0, 1])



import numpy as np

from sklearn.datasets import load_iris
from sklearn.utils.testing import assert_equal
from sklearn.utils.testing import assert_almost_equal

from lightning.classification import AdaGradClassifier
from lightning.regression import AdaGradRegressor
from lightning.impl.adagrad_fast import _proj_elastic_all
from lightning.impl.tests.utils import check_predict_proba

iris = load_iris()
X, y = iris.data, iris.target

X_bin = X[y <= 1]
y_bin = y[y <= 1] * 2 - 1


def test_adagrad_elastic_hinge():
    clf = AdaGradClassifier(alpha=0.5, l1_ratio=0.85, n_iter=10, random_state=0)
    clf.fit(X_bin, y_bin)
    assert not hasattr(clf, "predict_proba")
    assert_equal(clf.score(X_bin, y_bin), 1.0)


def test_adagrad_elastic_smooth_hinge():
    clf = AdaGradClassifier(alpha=0.5, l1_ratio=0.85, loss="smooth_hinge",
                            n_iter=10, random_state=0)
    clf.fit(X_bin, y_bin)
    assert not hasattr(clf, "predict_proba")
    assert_equal(clf.score(X_bin, y_bin), 1.0)


def test_adagrad_elastic_log():
    clf = AdaGradClassifier(alpha=0.1, l1_ratio=0.85, loss="log", n_iter=10,
                            random_state=0)
    clf.fit(X_bin, y_bin)
    assert_equal(clf.score(X_bin, y_bin), 1.0)
    check_predict_proba(clf, X_bin)


def test_adagrad_hinge_multiclass():
    clf = AdaGradClassifier(alpha=1e-2, n_iter=100, loss="hinge", random_state=0)
    clf.fit(X, y)
    assert not hasattr(clf, "predict_proba")
    assert_almost_equal(clf.score(X, y), 0.940, 3)


def test_adagrad_classes_binary():
    clf = AdaGradClassifier()
    assert not hasattr(clf, 'classes_')
    clf.fit(X_bin, y_bin)
    assert_equal(list(clf.classes_), [-1, 1])


def test_adagrad_classes_multiclass():
    clf = AdaGradClassifier()
    assert not hasattr(clf, 'classes_')
    clf.fit(X, y)
    assert_equal(list(clf.classes_), [0, 1, 2])


def test_adagrad_callback():
    class Callback(object):

        def __init__(self, X, y):
            self.X = X
            self.y = y
            self.acc = []

        def __call__(self, clf, t):
            alpha1 = clf.l1_ratio * clf.alpha
            alpha2 = (1 - clf.l1_ratio) * clf.alpha
            _proj_elastic_all(clf.eta, t, clf.g_sum_[0], clf.g_norms_[0],
                              alpha1, alpha2, 0, clf.coef_[0])
            score = clf.score(self.X, self.y)
            self.acc.append(score)

    cb = Callback(X_bin, y_bin)
    clf = AdaGradClassifier(alpha=0.5, l1_ratio=0.85, n_iter=10,
                            callback=cb, random_state=0)
    clf.fit(X_bin, y_bin)
    assert_equal(cb.acc[-1], 1.0)


def test_adagrad_regression():
    for loss in ("squared", "absolute"):
        reg = AdaGradRegressor(loss=loss)
        reg.fit(X_bin, y_bin)
        y_pred = np.sign(reg.predict(X_bin))
        assert_equal(np.mean(y_bin == y_pred), 1.0)



import pickle
import numpy as np
import scipy.sparse as sp

from sklearn.utils.testing import assert_array_equal
from sklearn.utils.testing import assert_array_almost_equal
from sklearn.utils.testing import assert_equal
from six.moves import xrange

from sklearn.datasets.samples_generator import make_classification
from sklearn.utils import check_random_state

from lightning.impl.dataset_fast import ContiguousDataset
from lightning.impl.dataset_fast import FortranDataset
from lightning.impl.dataset_fast import CSRDataset
from lightning.impl.dataset_fast import CSCDataset

# Create test datasets.
X, _ = make_classification(n_samples=20, n_features=100,
                           n_informative=5, n_classes=2, random_state=0)
X2, _ = make_classification(n_samples=10, n_features=100,
                            n_informative=5, n_classes=2, random_state=0)

# Sparsify datasets.
X[X < 0.3] = 0

X_csr = sp.csr_matrix(X)
X_csc = sp.csc_matrix(X)

rs = check_random_state(0)
cds = ContiguousDataset(X)
fds = FortranDataset(np.asfortranarray(X))
csr_ds = CSRDataset(X_csr)
csc_ds = CSCDataset(X_csc)


def test_contiguous_get_row():
    ind = np.arange(X.shape[1])
    for i in xrange(X.shape[0]):
        indices, data, n_nz = cds.get_row(i)
        assert_array_equal(indices, ind)
        assert_array_equal(data, X[i])
        assert_equal(n_nz, X.shape[1])


def test_csr_get_row():
    for i in xrange(X.shape[0]):
        indices, data, n_nz = csr_ds.get_row(i)
        for jj in xrange(n_nz):
            j = indices[jj]
            assert_equal(X[i, j], data[jj])


def test_fortran_get_column():
    ind = np.arange(X.shape[0])
    for j in xrange(X.shape[1]):
        indices, data, n_nz = fds.get_column(j)
        assert_array_equal(indices, ind)
        assert_array_equal(data, X[:, j])
        assert_equal(n_nz, X.shape[0])


def test_csc_get_column():
    for j in xrange(X.shape[1]):
        indices, data, n_nz = csc_ds.get_column(j)
        for ii in xrange(n_nz):
            i = indices[ii]
            assert_equal(X[i, j], data[ii])


def test_picklable_datasets():
    """Test that the datasets are picklable."""

    for dataset in [cds, csr_ds, fds, csc_ds]:
        pds = pickle.dumps(dataset)
        dataset = pickle.loads(pds)
        assert_equal(dataset.get_n_samples(), X.shape[0])
        assert_equal(dataset.get_n_features(), X.shape[1])



import numpy as np

from sklearn.datasets import load_iris
from sklearn.base import BaseEstimator, ClassifierMixin
from sklearn.utils.testing import assert_true
from sklearn.utils.testing import assert_equal

from lightning.classification import SVRGClassifier
from lightning.regression import SVRGRegressor
from lightning.impl.sgd_fast import SmoothHinge

iris = load_iris()
X, y = iris.data, iris.target

X_bin = X[y <= 1]
y_bin = y[y <= 1] * 2 - 1


def test_svrg():
    clf = SVRGClassifier(eta=1e-3, max_iter=20, random_state=0, verbose=0)
    clf.fit(X_bin, y_bin)
    assert not hasattr(clf, 'predict_proba')
    assert_equal(clf.score(X_bin, y_bin), 1.0)


def test_svrg_callback():
    class Callback(object):

        def __init__(self, X, y):
            self.X = X
            self.y = y
            self.obj = []

        def __call__(self, clf):
            clf._finalize_coef()
            y_pred = clf.decision_function(self.X).ravel()
            loss = (np.maximum(1 - self.y * y_pred, 0) ** 2).mean()
            coef = clf.coef_.ravel()
            regul = 0.5 * clf.alpha * np.dot(coef, coef)
            self.obj.append(loss + regul)

    cb = Callback(X_bin, y_bin)
    clf = SVRGClassifier(loss="squared_hinge", eta=1e-3, max_iter=20,
                         random_state=0, callback=cb)
    clf.fit(X_bin, y_bin)
    assert_true(np.all(np.diff(cb.obj) <= 0))


def test_svrg_regression():
    reg = SVRGRegressor(eta=1e-3)
    reg.fit(X_bin, y_bin)
    y_pred = np.sign(reg.predict(X_bin))
    assert_equal(np.mean(y_bin == y_pred), 1.0)


def test_bin_classes():
    clf = SVRGClassifier()
    clf.fit(X_bin, y_bin)
    assert_equal(list(clf.classes_), [-1, 1])


def test_multiclass_classes():
    clf = SVRGClassifier()
    clf.fit(X, y)
    assert_equal(list(clf.classes_), [0, 1, 2])



# Author: Mathieu Blondel
# License: BSD

import numpy as np

from sklearn.preprocessing import LabelEncoder
from sklearn.utils.extmath import safe_sparse_dot
from sklearn.metrics.pairwise import pairwise_kernels

from .base import BaseEstimator
from .dataset_fast import get_dataset
from .prank_fast import _prank_fit
from .prank_fast import _prank_fit_kernel
from .prank_fast import _prank_predict


class _BasePRank(BaseEstimator):

    def score(self, X, y):
        y_pred = self.predict(X)
        return np.mean(np.abs(y - y_pred))

    @property
    def classes_(self):
        return self._label_encoder.classes_


class PRank(_BasePRank):
    """Online algorithm for learning an ordinal regression model.

    Parameters
    ----------
    n_iter : int
        Number of iterations to run.

    shuffle : boolean
        Whether to shuffle data.

    random_state : RandomState or int
        The seed of the pseudo random number generator to use.

    Attributes
    ----------
    coef_ : array, shape=[n_features]
        Estimated weights.

    thresholds_ : array, shape=[n_classes]
        Estimated thresholds.

    Reference
    ---------
    Pranking with Ranking
    Koby Crammer, Yoram Singer
    NIPS 2001
    """

    def __init__(self, n_iter=10, shuffle=True, random_state=None):
        self.n_iter = n_iter
        self.shuffle = shuffle
        self.random_state = random_state

    def fit(self, X, y):
        """Fit model according to X and y.

        Parameters
        ----------
        X : array-like, shape = [n_samples, n_features]
            Training vectors, where n_samples is the number of samples
            and n_features is the number of features.

        y : array-like, shape = [n_samples]
            Target values.

        Returns
        -------
        self : classifier
            Returns self.
        """
        n_samples, n_features = X.shape
        rs = self._get_random_state()

        self._label_encoder = LabelEncoder()
        y = self._label_encoder.fit_transform(y).astype(np.int32)
        n_classes = len(self.classes_)


        self.coef_ = np.zeros(n_features, dtype=np.float64)
        self.thresholds_ = np.zeros(n_classes, dtype=np.float64)
        self.thresholds_[-1] = np.inf

        ds = get_dataset(X)

        _prank_fit(self.coef_, self.thresholds_, ds, y, n_classes,
                   self.n_iter, rs, self.shuffle)

        return self

    def predict(self, X):
        n_samples = X.shape[0]
        dot = safe_sparse_dot(X, self.coef_)
        out = np.zeros(n_samples, dtype=np.int32)
        _prank_predict(dot, self.thresholds_, len(self.classes_), out)
        return self._label_encoder.inverse_transform(out)


class KernelPRank(_BasePRank):
    """Kernelized online algorithm for learning an ordinal regression model.

    Parameters
    ----------
    n_iter : int
        Number of iterations to run.

    shuffle : boolean
        Whether to shuffle data.

    random_state : RandomState or int
        The seed of the pseudo random number generator to use.

    kernel: "linear" | "poly" | "rbf" | "sigmoid" | "cosine" | "precomputed"
        Kernel.
        Default: "linear"

    degree : int, default=3
        Degree for poly kernels. Ignored by other kernels.

    gamma : float, optional
        Kernel coefficient for rbf and poly kernels. Default: 1/n_features.
        Ignored by other kernels.

    coef0 : float, optional
        Independent term in poly and sigmoid kernels.
        Ignored by other kernels.

    kernel_params : mapping of string to any, optional
        Parameters (keyword arguments) and values for kernel passed as
        callable object. Ignored by other kernels.

    Attributes
    ----------
    dual_coef_ : array, shape=[n_samples]
        Estimated weights.

    thresholds_ : array, shape=[n_classes]
        Estimated thresholds.

    Reference
    ---------
    Pranking with Ranking
    Koby Crammer, Yoram Singer
    NIPS 2001
    """

    def __init__(self, n_iter=10, shuffle=True, random_state=None,
                 kernel="linear", gamma=None, degree=3, coef0=1,
                 kernel_params=None):
        self.n_iter = n_iter
        self.shuffle = shuffle
        self.random_state = random_state
        self.kernel = kernel
        self.gamma = gamma
        self.degree = degree
        self.coef0 = coef0
        self.kernel_params = kernel_params

    @property
    def _pairwise(self):
        return self.kernel == "precomputed"

    def _get_kernel(self, X, Y=None):
        if callable(self.kernel):
            params = self.kernel_params or {}
        else:
            params = {"gamma": self.gamma,
                      "degree": self.degree,
                      "coef0": self.coef0}
        return pairwise_kernels(X, Y, metric=self.kernel,
                                filter_params=True, **params)

    def fit(self, X, y):
        """Fit model according to X and y.

        Parameters
        ----------
        X : array-like, shape = [n_samples, n_features]
            Training vectors, where n_samples is the number of samples
            and n_features is the number of features.

        y : array-like, shape = [n_samples]
            Target values.

        Returns
        -------
        self : classifier
            Returns self.
        """
        n_samples, n_features = X.shape
        rs = self._get_random_state()

        self._label_encoder = LabelEncoder()
        y = self._label_encoder.fit_transform(y).astype(np.int32)
        n_classes = len(self.classes_)

        K = self._get_kernel(X)
        self.X_train_ = X

        self.dual_coef_ = np.zeros(n_samples, dtype=np.float64)
        self.thresholds_ = np.zeros(n_classes, dtype=np.float64)
        self.thresholds_[-1] = np.inf

        _prank_fit_kernel(self.dual_coef_, self.thresholds_, K, y, n_classes,
                   self.n_iter, rs, self.shuffle)

        return self

    def predict(self, X):
        K = self._get_kernel(X, self.X_train_)
        n_samples = X.shape[0]
        dot = np.dot(K, self.dual_coef_)
        out = np.zeros(n_samples, dtype=np.int32)
        _prank_predict(dot, self.thresholds_, len(self.classes_), out)
        return self._label_encoder.inverse_transform(out)



import pickle
import numpy as np
from numpy.testing import (assert_almost_equal, assert_array_equal,
                           assert_equal)
from lightning.impl.randomkit import RandomState
from six.moves import xrange


def test_randint():
    rs = RandomState(seed=0)
    vals = [rs.randint(10) for t in xrange(10000)]
    assert_almost_equal(np.mean(vals), 5.018)


def test_shuffle():
    ind = np.arange(10)
    rs = RandomState(seed=0)
    rs.shuffle(ind)
    assert_array_equal(ind, [2, 8, 4, 9, 1, 6, 7, 3, 0, 5])


def test_random_state_pickle():
    rs = RandomState(seed=0)
    random_integer = rs.randint(5)
    pickle_rs = pickle.dumps(rs)
    pickle_rs = pickle.loads(pickle_rs)
    pickle_random_integer = pickle_rs.randint(5)
    assert_equal(random_integer, pickle_random_integer)



import numpy as np

from sklearn.utils.testing import (assert_almost_equal,
                                   assert_array_almost_equal)
from six.moves import xrange

from lightning.impl.penalty import project_l1_ball, project_simplex


def project_simplex_bisection(v, z=1, tau=0.0001, max_iter=1000):
    lower = 0
    upper = np.max(v)
    current = np.inf

    for it in xrange(max_iter):
        if np.abs(current) / z < tau and current < 0:
            break

        theta = (upper + lower) / 2.0
        w = np.maximum(v - theta, 0)
        current = np.sum(w) - z
        if current <= 0:
            upper = theta
        else:
            lower = theta
    return w


def test_proj_simplex():
    rng = np.random.RandomState(0)

    v = rng.rand(100)
    w = project_simplex(v, z=10)
    w2 = project_simplex_bisection(v, z=10, max_iter=100)
    assert_array_almost_equal(w, w2, 3)

    v = rng.rand(3)
    w = project_simplex(v, z=1)
    w2 = project_simplex_bisection(v, z=1, max_iter=100)
    assert_array_almost_equal(w, w2, 3)

    v = rng.rand(2)
    w = project_simplex(v, z=1)
    w2 = project_simplex_bisection(v, z=1, max_iter=100)
    assert_array_almost_equal(w, w2, 3)


def test_proj_l1_ball():
    rng = np.random.RandomState(0)
    v = rng.randn(100)
    w = project_l1_ball(v, z=50)
    assert_almost_equal(np.sum(np.abs(w)), 50)
